@@ -1,9 +1,9 @@
 /*
- * Copyright 2003-2012 Gentoo Foundation
+ * Copyright 2003-2024 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  *
  * Copyright 2005-2012 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2012 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2024 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #include "paxinc.h"
@@ -86,7 +86,7 @@ static pairtype elf_etypes[] = {
 	{ 0, 0 }
 };
 
-unsigned int get_etype(elfobj *elf)
+unsigned int get_etype(const elfobj *elf)
 {
 	if (elf->elf_class == ELFCLASS32)
 		return EGET(EHDR32(elf->ehdr)->e_type);
@@ -94,12 +94,12 @@ unsigned int get_etype(elfobj *elf)
 		return EGET(EHDR64(elf->ehdr)->e_type);
 }
 
-const char *get_elfetype(elfobj *elf)
+const char *get_elfetype(const elfobj *elf)
 {
 	return find_pairtype(elf_etypes, get_etype(elf));
 }
 
-const char *get_endian(elfobj *elf)
+const char *get_endian(const elfobj *elf)
 {
 	switch (elf->data[EI_DATA]) {
 		case ELFDATA2LSB: return "LE";
@@ -109,7 +109,7 @@ const char *get_endian(elfobj *elf)
 }
 
 /* translate elf EF_ defines -- tricky as it's based on EM_ */
-static unsigned int get_eflags(elfobj *elf)
+static unsigned int get_eflags(const elfobj *elf)
 {
 	if (elf->elf_class == ELFCLASS32)
 		return EGET(EHDR32(elf->ehdr)->e_flags);
@@ -117,7 +117,7 @@ static unsigned int get_eflags(elfobj *elf)
 		return EGET(EHDR64(elf->ehdr)->e_flags);
 }
 
-static int arm_eabi_poker(elfobj *elf)
+static int arm_eabi_poker(const elfobj *elf)
 {
 	unsigned int emachine, eflags;
 
@@ -133,7 +133,7 @@ static int arm_eabi_poker(elfobj *elf)
 		return -1;
 }
 
-const char *get_elf_eabi(elfobj *elf)
+const char *get_elf_eabi(const elfobj *elf)
 {
 	static char buf[26];
 	int eabi = arm_eabi_poker(elf);
@@ -144,7 +144,7 @@ const char *get_elf_eabi(elfobj *elf)
 	return buf;
 }
 
-const char *get_elfosabi(elfobj *elf)
+const char *get_elfosabi(const elfobj *elf)
 {
 	const char *str = get_elfeitype(EI_OSABI, elf->data[EI_OSABI]);
 	if (strncmp(str, "ELFOSABI_", 9) == 0)
@@ -276,10 +276,12 @@ static pairtype elf_emtypes[] = {
 	QUERY(EM_MICROBLAZE),
 	QUERY(EM_TILEGX),
 	QUERY(EM_ALPHA),
+	QUERY(EM_RISCV),
+	QUERY(EM_LOONGARCH),
 	{ 0, 0 }
 };
 
-unsigned int get_emtype(elfobj *elf)
+unsigned int get_emtype(const elfobj *elf)
 {
 	if (elf->elf_class == ELFCLASS32)
 		return EGET(EHDR32(elf->ehdr)->e_machine);
@@ -287,7 +289,7 @@ unsigned int get_emtype(elfobj *elf)
 		return EGET(EHDR64(elf->ehdr)->e_machine);
 }
 
-const char *get_elfemtype(elfobj *elf)
+const char *get_elfemtype(const elfobj *elf)
 {
 	return find_pairtype(elf_emtypes, get_emtype(elf));
 }
@@ -558,7 +560,7 @@ const char *get_elfnttype(uint16_t e_type, const char *name, int type)
 	((buff[EI_CLASS] == ELFCLASS32 || buff[EI_CLASS] == ELFCLASS64) && \
 	 (buff[EI_DATA] == ELFDATA2LSB || buff[EI_DATA] == ELFDATA2MSB) && \
 	 (buff[EI_VERSION] == EV_CURRENT))
-elfobj *readelf_buffer(const char *filename, void *buffer, size_t buffer_len)
+elfobj *readelf_buffer(const char *filename, const void *buffer, size_t buffer_len)
 {
 	elfobj *elf;
 
@@ -582,7 +584,7 @@ free_elf_and_return:
 
 	/* check class and stuff */
 	if (!DO_WE_LIKE_ELF(elf->data)) {
-		warn("we no likey %s: {%s,%s,%s,%s}",
+		warn("unknown ELF settings: %s: {%s,%s,%s,%s}",
 		     filename,
 		     get_elfeitype(EI_CLASS, elf->data[EI_CLASS]),
 		     get_elfeitype(EI_DATA, elf->data[EI_DATA]),
@@ -616,8 +618,13 @@ free_elf_and_return:
 #define READELF_HEADER(B) \
 	if (elf->elf_class == ELFCLASS ## B) { \
 		char invalid; \
-		Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
+		const Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
 		Elf ## B ## _Off size; \
+		/* Need enough bytes for all of ehdr. */ \
+		if (elf->len < (off_t)sizeof(*ehdr)) { \
+			warn("%s: Incomplete ELF header", filename); \
+			goto free_elf_and_return; \
+		} \
 		/* verify program header */ \
 		invalid = 0; \
 		if (EGET(ehdr->e_phnum) <= 0) \
@@ -677,7 +684,7 @@ elfobj *_readelf_fd(const char *filename, int fd, size_t len, int read_only)
 
 	buffer = mmap(0, len, PROT_READ | (read_only ? 0 : PROT_WRITE), (read_only ? MAP_PRIVATE : MAP_SHARED), fd, 0);
 	if (buffer == MAP_FAILED) {
-		warn("mmap on '%s' of %li bytes failed :(", filename, (unsigned long)len);
+		warn("mmap on '%s' of %zu bytes failed :(", filename, len);
 		return NULL;
 	}
 
@@ -720,7 +727,7 @@ elfobj *_readelf(const char *filename, int read_only)
 /* undo the readelf() stuff */
 void unreadelf(elfobj *elf)
 {
-	if (elf->is_mmap) munmap(elf->vdata, elf->len);
+	if (elf->is_mmap) munmap((void *)elf->vdata, elf->len);
 	if (elf->fd != -1) close(elf->fd);
 	if (!__PAX_UNALIGNED_OK) free(elf->_data);
 	free(elf);
@@ -784,19 +791,19 @@ const char *gnu_short_stack_flags(unsigned long flags)
 	return buffer;
 }
 
-void *elf_findsecbyname(elfobj *elf, const char *name)
+const void *elf_findsecbyname(const elfobj *elf, const char *name)
 {
 	unsigned int i;
-	char *shdr_name;
-	void *ret = NULL;
+	const char *shdr_name;
+	const void *ret = NULL;
 
 	if (elf->shdr == NULL) return NULL;
 
 #define FINDSEC(B) \
 	if (elf->elf_class == ELFCLASS ## B) { \
-	Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
-	Elf ## B ## _Shdr *shdr = SHDR ## B (elf->shdr); \
-	Elf ## B ## _Shdr *strtbl; \
+	const Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
+	const Elf ## B ## _Shdr *shdr = SHDR ## B (elf->shdr); \
+	const Elf ## B ## _Shdr *strtbl; \
 	Elf ## B ## _Off offset; \
 	uint16_t shstrndx = EGET(ehdr->e_shstrndx); \
 	uint16_t shnum = EGET(ehdr->e_shnum); \
@@ -805,10 +812,10 @@ void *elf_findsecbyname(elfobj *elf, const char *name)
 	for (i = 0; i < shnum; ++i) { \
 		offset = EGET(strtbl->sh_offset) + EGET(shdr[i].sh_name); \
 		if (offset >= (Elf ## B ## _Off)elf->len) continue; \
-		shdr_name = elf->data + offset; \
+		shdr_name = elf->cdata + offset; \
 		if (!strcmp(shdr_name, name)) { \
 			if (ret) warnf("Multiple '%s' sections !?", name); \
-			ret = (void*)&(shdr[i]); \
+			ret = &shdr[i]; \
 		} \
 	} }
 	FINDSEC(32)
