@@ -1,10 +1,10 @@
 /*
- * Copyright 2003-2012 Gentoo Foundation
+ * Copyright 2003-2024 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  *
  * Copyright 2005-2012 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2012 Mike Frysinger  - <vapier@gentoo.org>
- *           2008-2012 Fabian Groffen  - <grobian@gentoo.org>
+ * Copyright 2005-2024 Mike Frysinger  - <vapier@gentoo.org>
+ *           2008-2021 Fabian Groffen  - <grobian@gentoo.org>
  */
 
 #include "paxinc.h"
@@ -17,6 +17,7 @@ static const char STR_PPC64[]   = "ppc64";
 static const char STR_I386[]    = "i386";
 static const char STR_X86_64[]  = "x86_64";
 static const char STR_ARM[]     = "arm"; /* iPhone */
+static const char STR_ARM64[]   = "arm64"; /* Apple M1 */
 static const char STR_UNKNOWN[] = "unknown";
 
 #define QUERY(n) { #n, n }
@@ -95,6 +96,7 @@ static pairtype macho_cputype[] = {
 	QUERY(CPU_TYPE_ARM),
 	QUERY(CPU_TYPE_POWERPC64),
 	QUERY(CPU_TYPE_X86_64),
+	QUERY(CPU_TYPE_ARM64),
 	{ 0, 0 }
 };
 const char *get_machocputype(fatobj *fobj)
@@ -164,6 +166,7 @@ inline static uint32_t read_mach_header(fatobj *fobj, void *addr)
 	switch (mhdr->magic) {
 		case MH_CIGAM:
 			fobj->swapped = 1;
+			/* fall through */
 		case MH_MAGIC:
 			/* 32-bits */
 			fobj->ismach64 = 0;
@@ -172,6 +175,7 @@ inline static uint32_t read_mach_header(fatobj *fobj, void *addr)
 			return mhdr->magic;
 		case MH_CIGAM_64:
 			fobj->swapped = 1;
+			/* fall through */
 		case MH_MAGIC_64:
 			/* 64-bits */
 			fobj->ismach64 = 1;
@@ -187,16 +191,16 @@ inline static uint32_t read_mach_header(fatobj *fobj, void *addr)
  * arch. */
 fatobj *readmacho(const char *filename)
 {
-	struct stat st;
 	int fd;
-
-	if (stat(filename, &st) == -1)
-		return NULL;
+	fatobj *ret;
 
 	if ((fd = open(filename, O_RDONLY)) == -1)
 		return NULL;
 
-	return readmacho_fd(filename, fd, st.st_size);
+	ret = readmacho_fd(filename, fd, 0);
+	if (ret == NULL)
+		close(fd);
+	return ret;
 }
 
 fatobj *readmacho_fd(const char *filename, int fd, size_t len)
@@ -245,7 +249,7 @@ fatobj *readmacho_buffer(const char *filename, char *buffer, size_t buffer_len)
 
 	/* make sure we have enough bytes to scan */
 	if (ret->len <= sizeof(struct fat_header))
-		return NULL;
+		goto fail;
 
 	fhdr = ret->data;
 	/* Check what kind of file this is.  Unfortunately we don't have
@@ -274,14 +278,14 @@ fatobj *readmacho_buffer(const char *filename, char *buffer, size_t buffer_len)
 		 * beware of corrupt files and Java bytecode which shares
 		 * the same magic with us :( */
 		if (sizeof(struct fat_arch) * narchs > bufleft)
-			return NULL;
+			goto fail;
 
 		for (i = 1; i <= narchs; i++) {
 			farch = (struct fat_arch *)dptr;
 			offset = MGET(swapped, farch->offset);
 			if (offset + sizeof(struct mach_header) >= bufleft ||
 					read_mach_header(fobj, ret->data + offset) == 0)
-				return NULL;
+				goto fail;
 			if (i < narchs) {
 				fobj = fobj->next = xzalloc(sizeof(*fobj));
 				/* filename and size are necessary for printing */
@@ -298,11 +302,15 @@ fatobj *readmacho_buffer(const char *filename, char *buffer, size_t buffer_len)
 		/* simple Mach-O file, treat as single arch FAT file */
 		if (ret->len < sizeof(struct mach_header) ||
 				read_mach_header(ret, ret->data) == 0)
-			return NULL;
+			goto fail;
 		ret->next = NULL;
 	}
 
 	return ret;
+
+ fail:
+	free(ret);
+	return NULL;
 }
 
 /* undo the readmacho() stuff */
@@ -377,6 +385,7 @@ const char *get_machomtype(fatobj *fobj)
 		case CPU_TYPE_ARM:       return STR_ARM;
 		case CPU_TYPE_POWERPC64: return STR_PPC64;
 		case CPU_TYPE_X86_64:    return STR_X86_64;
+		case CPU_TYPE_ARM64:     return STR_ARM64;
 		default:                 return STR_UNKNOWN;
 	}
 }
